import { useEffect, useCallback } from 'react';
import useAsyncReducer from 'use-async-reducer';
function getUpdaterPromise(asyncUpdater) {
    if (typeof asyncUpdater === 'function') {
        return asyncUpdater();
    }
    return asyncUpdater;
}
export default function useAsyncCall(asyncCreator, options = {}) {
    const [response, actions] = useAsyncReducer(options.initialValue);
    useEffect(() => {
        let didCancel = false;
        const callAsync = async () => {
            actions.request();
            try {
                const data = await asyncCreator();
                if (didCancel) {
                    return;
                }
                actions.success(data);
                if (typeof options.onSuccess === 'function') {
                    options.onSuccess(data);
                }
            }
            catch (error) {
                if (didCancel) {
                    return;
                }
                actions.failure(error);
                if (typeof options.onFailure === 'function') {
                    options.onFailure(error);
                }
            }
            finally {
                if (didCancel) {
                    return;
                }
                if (typeof options.onComplete === 'function') {
                    options.onComplete();
                }
            }
        };
        callAsync();
        return () => {
            didCancel = true;
        };
    }, [asyncCreator, options.onSuccess, options.onFailure, options.onComplete]);
    const update = useCallback(async (asyncUpdater, updateOptions = {}) => {
        // Placeholder while working out how to cancel updates when the update
        // method changes
        let updateDidCancel = false;
        actions.request();
        try {
            const data = await getUpdaterPromise(asyncUpdater);
            if (updateDidCancel) {
                return;
            }
            actions.success(data);
            if (typeof updateOptions.onSuccess === 'function') {
                updateOptions.onSuccess(data);
            }
            return data;
        }
        catch (error) {
            if (updateDidCancel) {
                return;
            }
            if (updateOptions.saveError) {
                actions.failure(error);
            }
            else {
                actions.complete();
            }
            if (typeof updateOptions.onFailure === 'function') {
                updateOptions.onFailure(error);
            }
            if (updateOptions.throwError) {
                throw error;
            }
        }
        finally {
            if (updateDidCancel) {
                return;
            }
            if (typeof updateOptions.onComplete === 'function') {
                updateOptions.onComplete();
            }
        }
    }, [actions]);
    return [response, update];
}
